<?php

namespace FiloBlu\VatValidatorApi\Controller\Index;

use FiloBlu\VatValidatorApi\Api\ValidationServiceRequestInterface;
use FiloBlu\VatValidatorApi\Api\ValidationServiceRequestInterfaceFactory;
use FiloBlu\VatValidatorApi\Api\VatValidatorServiceInterface;
use Magento\Framework\App\ActionInterface;
use Magento\Framework\Controller\Result\JsonFactory;

class Validate implements ActionInterface
{
    /**
     * @var ValidationServiceRequestInterfaceFactory
     */
    private $validationServiceRequestFactory;

    /**
     * @var VatValidatorServiceInterface
     */
    private $vatValidatorService;

    /**
     * @var \Magento\Framework\App\Action\Context
     */
    private $context;

    /**
     * @var JsonFactory
     */
    private $responseFactory;

    /**
     * @param \Magento\Framework\App\Action\Context $context
     * @param ValidationServiceRequestInterfaceFactory $validationServiceRequestFactory
     * @param VatValidatorServiceInterface $vatValidatorService
     * @param JsonFactory $responseFactory
     */
    public function __construct(
        \Magento\Framework\App\Action\Context $context,
        ValidationServiceRequestInterfaceFactory $validationServiceRequestFactory,
        VatValidatorServiceInterface $vatValidatorService,
        JsonFactory $responseFactory
    ) {
        $this->validationServiceRequestFactory = $validationServiceRequestFactory;
        $this->vatValidatorService = $vatValidatorService;
        $this->context = $context;
        $this->responseFactory = $responseFactory;
    }

    /**
     * @return \Magento\Framework\App\ResponseInterface|\Magento\Framework\Controller\Result\Json|\Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        $request = $this->context->getRequest();
        $response = $this->responseFactory->create();
        if (!$request->isXmlHttpRequest() && !$request->getParam('ajax') && !$request->getParam('isAjax')) {
            return $response->setData(['success' => false]);
        }

        $countryCode = $request->getParam(
            'country-code',
            $request->getParam(ValidationServiceRequestInterface::COUNTRY_CODE)
        );

        $vatNumber = $request->getParam(
            'vat-number',
            $request->getParam(ValidationServiceRequestInterface::VAT_NUMBER)
        );

        if (!$countryCode || !$vatNumber) {
            return $response->setData(['success' => false]);
        }

        $validationRequest = $this->validationServiceRequestFactory->create(['data' => [
            ValidationServiceRequestInterface::COUNTRY_CODE => $countryCode,
            ValidationServiceRequestInterface::VAT_NUMBER => $vatNumber
        ]]);

        $result = $this->vatValidatorService->isValid($validationRequest);

        return $response->setData([
            'success' => $result
        ]);
    }

}