<?php

namespace FiloBlu\VatValidatorApi\Helper;

use FiloBlu\VatValidatorApi\Api\ValidationServiceRequestInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;

class VatHelper
{
    const EU_COUNTRY_LIST = 'general/country/eu_countries';

    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;

    /**
     * @param ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig
    ) {
        $this->scopeConfig = $scopeConfig;
    }

    /**
     * @param string $countryCode
     * @return bool
     */
    public function isEUCountry(string $countryCode): bool
    {
        $euCountries = explode(',', $this->scopeConfig->getValue(self::EU_COUNTRY_LIST));
        return \in_array($countryCode, $euCountries);
    }

    /**
     * @param string $countryCode
     * @return bool
     */
    public function isUKCountry(string $countryCode): bool
    {
        return \substr($countryCode, 0, 2) === 'GB';
    }

    /**
     * @param string $countryCode
     * @return bool
     */
    public function isCHECountry(string $countryCode): bool
    {
        return \substr($countryCode, 0, 2) === 'CH';
    }

    /**
     * @param string $countryCode
     * @return string
     */
    public function formatCountryCode(string $countryCode): string
    {
        return $countryCode === 'GR' ? 'EL' : $countryCode;
    }

    /**
     * @param ValidationServiceRequestInterface $request
     * @return string
     */
    public function formatVatNumber(ValidationServiceRequestInterface $request): string
    {
        $vatNumber = $request->getVatNumber();
        $countryCode = $request->getCountryCode();

        if (substr($vatNumber, 0, strlen($countryCode)) === $countryCode) {
            $vatNumber = substr($vatNumber, strlen($countryCode));
        }

        return $vatNumber;
    }
}