<?php

declare(strict_types=1);

namespace FiloBlu\VatValidatorApi\Model;

use FiloBlu\VatValidatorApi\Api\ValidationServiceInterface;
use FiloBlu\VatValidatorApi\Api\ValidationServiceRequestInterface;
use Magento\Framework\HTTP\ZendClientFactory;
use Zend_Http_Client;

class EuValidationService implements ValidationServiceInterface
{
    const VIES_BASE_URL = 'https://ec.europa.eu/taxation_customs/vies/rest-api//';
    const VIES_CHECK_VAT_NUMBER = 'check-vat-number';

    /**
     * @var ZendClientFactory
     */
    private $httpClientFactory;

    /**
     * @var ValidationResult
     */
    private $validationResult;

    /**
     * @param ZendClientFactory $httpClientFactory
     * @param ValidationResult $validationResult
     */
    public function __construct(ZendClientFactory $httpClientFactory, ValidationResult $validationResult)
    {
        $this->httpClientFactory = $httpClientFactory;
        $this->validationResult = $validationResult;
    }

    /**
     * @param \FiloBlu\VatValidatorApi\Api\ValidationServiceRequestInterface $request
     * @return \FiloBlu\VatValidatorApi\Model\ValidationResult
     * @throws \Zend_Http_Client_Exception
     */
    public function validate(ValidationServiceRequestInterface $request): ValidationResult
    {
        $url = EuValidationService::VIES_BASE_URL . EuValidationService::VIES_CHECK_VAT_NUMBER;
        $client = $this->httpClientFactory->create();
        $client->setUri($url);
        $client->setMethod(Zend_Http_Client::POST);
        $client->setHeaders(Zend_Http_Client::CONTENT_TYPE, 'application/json');
        $client->setHeaders('Accept', 'application/json');

        $countryCode = $request->getCountryCode();

        if ($countryCode === 'GR') {
            $countryCode = 'EL' ;
        }

        $data = [
            'countryCode' => $countryCode ,
            'vatNumber'   => $request->getVatNumber(),
        ];

        $client->setRawData(json_encode($data));
        $response = $client->request();

        return $this->formatResponse($response);
    }

    /**
     * @param $response
     * @return ValidationResult
     */
    public function formatResponse($response): ValidationResult
    {
        $body = json_decode($response->getBody(), true);
        $responseStatus = $response->getStatus();

        if ($responseStatus === 200) {
            if (array_key_exists('valid', $body)) {
                $this->validationResult->setIsValid($body['valid']);
                return $this->validationResult;
            }
            if (array_key_exists('actionSucceed', $body) && !$body['actionSucceed']) {
                $this->validationResult->setIsValid(true);
            }
        }

        if ($responseStatus === 400 || $responseStatus === 500) {
            $error = array_key_exists('errorWrappers', $body) ? $body['errorWrappers'] : [];
            $this->validationResult->setError($error);

            return $this->validationResult;
        }

        return $this->validationResult;
    }
}
