<?php

declare(strict_types=1);

namespace FiloBlu\VatValidatorApi\Model;

use FiloBlu\VatValidatorApi\Api\ValidationServiceInterface;
use FiloBlu\VatValidatorApi\Api\ValidationServiceRequestInterface;
use Magento\Framework\HTTP\ZendClientFactory;
use Zend_Http_Client;

class UkValidationService implements ValidationServiceInterface
{
    const UK_BASE_URL = 'https://api.service.hmrc.gov.uk/organisations/vat/check-vat-number/lookup/';

    /**
     * @var ZendClientFactory
     */
    private $httpClientFactory;

    /**
     * @var ValidationResultFactory
     */
    private $validationResultFactory;

    /**
     * @param ZendClientFactory $httpClientFactory
     * @param ValidationResultFactory $validationResultFactory
     */
    public function __construct(ZendClientFactory $httpClientFactory, ValidationResultFactory $validationResultFactory)
    {
        $this->httpClientFactory = $httpClientFactory;
        $this->validationResultFactory = $validationResultFactory;
    }

    /**
     * @param ValidationServiceRequestInterface $request
     * @return ValidationResult
     * @throws \Zend_Http_Client_Exception
     */
    public function validate(ValidationServiceRequestInterface $request): ValidationResult
    {
        $url = UkValidationService::UK_BASE_URL . $request->getVatNumber();

        $client = $this->httpClientFactory->create();
        $client->setUri($url);
        $client->setHeaders(Zend_Http_Client::CONTENT_TYPE, 'application/json');
        $client->setHeaders('Accept', 'application/json');

        $response = $client->request();
        return $this->formatResponse($response);
    }

    /**
     * @param $response
     * @return ValidationResult
     */
    public function formatResponse($response): ValidationResult
    {
        $validationResult = $this->validationResultFactory->create();

        if ($response->getStatus() === 200) {
            return $validationResult;
        }

        $validationResult->setIsValid(false);
        $validationResult->setError([
            'code'    => $response->getStatus(),
            'message' => $response->getMessage()
        ]);

        return $validationResult;
    }
}
