<?php

namespace FiloBlu\VatValidatorApi\Model;

use FiloBlu\VatValidatorApi\Api\ValidationServiceInterface;
use FiloBlu\VatValidatorApi\Api\ValidationServiceRequestInterface;
use FiloBlu\VatValidatorApi\Api\ValidationServiceRequestInterfaceFactory;
use FiloBlu\VatValidatorApi\Helper\VatHelper;
use function strlen;

class Validator
{
    /**
     * @var VatHelper
     */
    private $vatHelper;

    /**
     * @var ValidationResult
     */
    private $validationResultFactory;

    /**
     * @var \FiloBlu\VatValidatorApi\Api\ValidationServiceInterface
     */
    private $euValidator;

    /**
     * @var \FiloBlu\VatValidatorApi\Api\ValidationServiceInterface
     */
    private $ukValidator;

    /**
     * @param VatHelper $vatHelper
     * @param ValidationResultFactory $validationResultFactory
     * @param ValidationServiceInterface $euValidator
     * @param ValidationServiceInterface $ukValidator
     */
    public function __construct(
        VatHelper $vatHelper,
        ValidationResultFactory $validationResultFactory,
        ValidationServiceInterface $euValidator,
        ValidationServiceInterface $ukValidator
    ) {
        $this->vatHelper = $vatHelper;
        $this->validationResultFactory = $validationResultFactory;
        $this->euValidator = $euValidator;
        $this->ukValidator = $ukValidator;
    }

    /**
     * @param ValidationServiceRequestInterface $request
     * @return ValidationResult
     */
    public function checkInputData(ValidationServiceRequestInterface $request): ValidationResult
    {
        $vatNumber = $request->getVatNumber();
        $countryCode = $request->getCountryCode();

        $validationResult = $this->validationResultFactory->create();
        if ($vatNumber === '' || $countryCode === '') {
            $validationResult->setIsValid(false);
            $validationResult->setError(
                [
                    'code'    => '',
                    'message' => __('Invalid input data')
                ]
            );
            return $validationResult;
        }

        if (strlen($vatNumber) > 20) {
            $validationResult->setIsValid(false);
            return $validationResult;
        }

        if (strlen($countryCode) > 3) {
            $validationResult->setIsValid(false);
            return $validationResult;
        }

        return $validationResult;
    }

    /**
     * @param ValidationServiceRequestInterface $request
     * @return ValidationResult
     */
    public function validate(ValidationServiceRequestInterface $request): ValidationResult
    {
        $request->setVatNumber($this->vatHelper->formatVatNumber($request));
        $countryCode = $request->getCountryCode();

        $validationResult = $this->validationResultFactory->create();

        if ($this->vatHelper->isCHECountry($countryCode)) {
            return $validationResult;
        }

        if ($this->vatHelper->isUKCountry($countryCode)) {
            return $this->ukValidator->validate($request);
        }

        if ($this->vatHelper->isEUCountry($countryCode)) {
            return $this->euValidator->validate($request);
        }

        $validationResult->setIsValid(false);
        return $validationResult;
    }
}
